@@ -25,8 +25,9 @@
  *====================================================================*/
 
 
-/*
- *  fhmtauto.c
+/*!
+ * \file fhmtauto.c
+ * <pre>
  *
  *    Main function calls:
  *       l_int32             fhmtautogen()
@@ -86,8 +87,13 @@
  *        verifies the correctness of the implementation by
  *        comparing the dwa result with that of full-image
  *        rasterops.
+ * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -97,7 +103,7 @@
 
 #define   PROTOARGS   "(l_uint32 *, l_int32, l_int32, l_int32, l_uint32 *, l_int32);"
 
-static const l_int32  L_BUF_SIZE = 512;
+#define L_BUF_SIZE 512
 
 static char * makeBarrelshiftString(l_int32 delx, l_int32 dely, l_int32 type);
 static SARRAY * sarrayMakeInnerLoopDWACode(SEL *sel, l_int32 nhits, l_int32 nmisses);
@@ -187,19 +193,21 @@ static char wplstrm[][10] = {"- wpls", "- wpls2", "- wpls3", "- wpls4",
 
 
 /*!
- *  fhmtautogen()
+ * \brief   fhmtautogen()
  *
- *      Input:  sela
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function generates all the code for implementing
  *          dwa morphological operations using all the sels in the sela.
  *      (2) See fhmtautogen1() and fhmtautogen2() for details.
+ * </pre>
  */
-l_int32
+l_ok
 fhmtautogen(SELA        *sela,
             l_int32      fileindex,
             const char  *filename)
@@ -219,27 +227,29 @@ l_int32  ret1, ret2;
 
 
 /*!
- *  fhmtautogen1()
+ * \brief   fhmtautogen1()
  *
- *      Input:  sel array
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela array
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function uses hmttemplate1.txt to create a
  *          top-level file that contains two functions that carry
  *          out the hit-miss transform for any of the sels in
  *          the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fhmtgen.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>.<n>.c.
+ *      (3) If filename == NULL, the output file is fhmtgen.[n].c,
+ *          where [n] is equal to the 'fileindex' parameter.
+ *      (4) If filename != NULL, the output file is [filename].[n].c.
  *      (5) Each sel must have at least one hit.  A sel with only misses
  *          generates code that will abort the operation if it is called.
+ * </pre>
  */
-l_int32
+l_ok
 fhmtautogen1(SELA        *sela,
              l_int32      fileindex,
              const char  *filename)
@@ -263,15 +273,16 @@ SARRAY  *sa1, *sa2, *sa3;
     if ((nsels = selaGetCount(sela)) == 0)
         return ERROR_INT("no sels in sela", procName, 1);
 
-        /* Make array of sel names */
-    sa1 = selaGetSelnames(sela);
-
         /* Make array of textlines from from hmttemplate1.txt */
     if ((filestr = (char *)l_binaryRead(TEMPLATE1, &size)) == NULL)
         return ERROR_INT("filestr not made", procName, 1);
-    if ((sa2 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
+    sa2 = sarrayCreateLinesFromString(filestr, 1);
+    LEPT_FREE(filestr);
+    if (!sa2)
         return ERROR_INT("sa2 not made", procName, 1);
-    FREE(filestr);
+
+        /* Make array of sel names */
+    sa1 = selaGetSelnames(sela);
 
         /* Make strings containing function call names */
     sprintf(bigbuf, "PIX *pixHMTDwa_%d(PIX *pixd, PIX *pixs, "
@@ -289,9 +300,9 @@ SARRAY  *sa1, *sa2, *sa3;
     str_doc1 = stringNew(bigbuf);
     sprintf(bigbuf, " *             PIX     *pixFHMTGen_%d()", fileindex);
     str_doc2 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixHMTDwa_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief  pixHMTDwa_%d()", fileindex);
     str_doc3 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixFHMTGen_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief  pixFHMTGen_%d()", fileindex);
     str_doc4 = stringNew(bigbuf);
     sprintf(bigbuf, "pixHMTDwa_%d(PIX         *pixd,", fileindex);
     str_def1 = stringNew(bigbuf);
@@ -314,8 +325,7 @@ SARRAY  *sa1, *sa2, *sa3;
     str_low_ds = stringNew(bigbuf);
 
         /* Make the output sa */
-    if ((sa3 = sarrayCreate(0)) == NULL)
-        return ERROR_INT("sa3 not made", procName, 1);
+    sa3 = sarrayCreate(0);
 
         /* Copyright notice and info header */
     sarrayParseRange(sa2, 0, &actstart, &end, &newstart, "--", 0);
@@ -340,12 +350,12 @@ SARRAY  *sa1, *sa2, *sa3;
     sprintf(bigbuf, "static char  SEL_NAMES[][80] = {");
     sarrayAddString(sa3, bigbuf, L_COPY);
     for (i = 0; i < nsels - 1; i++) {
-        sprintf(bigbuf,
-        "                             \"%s\",", sarrayGetString(sa1, i, 0));
+        sprintf(bigbuf, "                             \"%s\",",
+                sarrayGetString(sa1, i, L_NOCOPY));
         sarrayAddString(sa3, bigbuf, L_COPY);
     }
-    sprintf(bigbuf,
-        "                             \"%s\"};", sarrayGetString(sa1, i, 0));
+    sprintf(bigbuf, "                             \"%s\"};",
+            sarrayGetString(sa1, i, L_NOCOPY));
     sarrayAddString(sa3, bigbuf, L_COPY);
 
         /* Start pixHMTDwa_*() function description */
@@ -385,42 +395,43 @@ SARRAY  *sa1, *sa2, *sa3;
     sarrayParseRange(sa2, newstart, &actstart, &end, &newstart, "--", 0);
     sarrayAppendRange(sa3, sa2, actstart, end);
 
-    if ((filestr = sarrayToString(sa3, 1)) == NULL)
-        return ERROR_INT("filestr from sa3 not made", procName, 1);
+    filestr = sarrayToString(sa3, 1);
     nbytes = strlen(filestr);
     if (filename)
-        sprintf(bigbuf, "%s.%d.c", filename, fileindex);
+        snprintf(bigbuf, L_BUF_SIZE, "%s.%d.c", filename, fileindex);
     else
         sprintf(bigbuf, "%s.%d.c", OUTROOT, fileindex);
     l_binaryWrite(bigbuf, "w", filestr, nbytes);
     sarrayDestroy(&sa1);
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
-    FREE(filestr);
+    LEPT_FREE(filestr);
     return 0;
 }
 
 
 /*!
- *  fhmtautogen2()
+ * \brief   fhmtautogen2()
  *
- *      Input:  sel array
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela array
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function uses hmttemplate2.txt to create a
  *          low-level file that contains the low-level functions for
  *          implementing the hit-miss transform for every sel
  *          in the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fhmtgenlow.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>low.<n>.c.
+ *      (3) If filename == NULL, the output file is fhmtgenlow.[n].c,
+ *          where [n] is equal to the %fileindex parameter.
+ *      (4) If filename != NULL, the output file is [filename]low.[n].c.
+ * </pre>
  */
-l_int32
+l_ok
 fhmtautogen2(SELA        *sela,
              l_int32      fileindex,
              const char  *filename)
@@ -449,25 +460,27 @@ SEL     *sel;
         /* Make the array of textlines from hmttemplate2.txt */
     if ((filestr = (char *)l_binaryRead(TEMPLATE2, &size)) == NULL)
         return ERROR_INT("filestr not made", procName, 1);
-    if ((sa1 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
+    sa1 = sarrayCreateLinesFromString(filestr, 1);
+    LEPT_FREE(filestr);
+    if (!sa1)
         return ERROR_INT("sa1 not made", procName, 1);
-    FREE(filestr);
 
         /* Make the array of static function names */
-    if ((sa2 = sarrayCreate(nsels)) == NULL)
+    if ((sa2 = sarrayCreate(nsels)) == NULL) {
+        sarrayDestroy(&sa1);
         return ERROR_INT("sa2 not made", procName, 1);
+    }
     for (i = 0; i < nsels; i++) {
         sprintf(bigbuf, "fhmt_%d_%d", fileindex, i);
-        sarrayAddString(sa2, bigbuf, 1);
+        sarrayAddString(sa2, bigbuf, L_COPY);
     }
 
         /* Make the static prototype strings */
-    if ((sa3 = sarrayCreate(2 * nsels)) == NULL)
-        return ERROR_INT("sa3 not made", procName, 1);
+    sa3 = sarrayCreate(2 * nsels);  /* should be ok */
     for (i = 0; i < nsels; i++) {
-        fname = sarrayGetString(sa2, i, 0);
+        fname = sarrayGetString(sa2, i, L_NOCOPY);
         sprintf(bigbuf, "static void  %s%s", fname, PROTOARGS);
-        sarrayAddString(sa3, bigbuf, 1);
+        sarrayAddString(sa3, bigbuf, L_COPY);
     }
 
         /* Make strings containing function names */
@@ -476,14 +489,9 @@ SEL     *sel;
     str_doc1 = stringNew(bigbuf);
     sprintf(bigbuf, " *             void       fhmt_%d_*()", fileindex);
     str_doc2 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  fhmtgen_low_%d()", fileindex);
-    str_doc3 = stringNew(bigbuf);
-    sprintf(bigbuf, "fhmtgen_low_%d(l_uint32  *datad,", fileindex);
-    str_def1 = stringNew(bigbuf);
 
         /* Output to this sa */
-    if ((sa4 = sarrayCreate(0)) == NULL)
-        return ERROR_INT("sa4 not made", procName, 1);
+    sa4 = sarrayCreate(0);
 
         /* Copyright notice and info header */
     sarrayParseRange(sa1, 0, &actstart, &end, &newstart, "--", 0);
@@ -499,11 +507,22 @@ SEL     *sel;
 
         /* Insert static protos */
     for (i = 0; i < nsels; i++) {
-        if ((linestr = sarrayGetString(sa3, i, L_COPY)) == NULL)
+        if ((linestr = sarrayGetString(sa3, i, L_COPY)) == NULL) {
+            sarrayDestroy(&sa1);
+            sarrayDestroy(&sa2);
+            sarrayDestroy(&sa3);
+            sarrayDestroy(&sa4);
             return ERROR_INT("linestr not retrieved", procName, 1);
+        }
         sarrayAddString(sa4, linestr, L_INSERT);
     }
 
+        /* Make more strings containing function names */
+    sprintf(bigbuf, " *  fhmtgen_low_%d()", fileindex);
+    str_doc3 = stringNew(bigbuf);
+    sprintf(bigbuf, "fhmtgen_low_%d(l_uint32  *datad,", fileindex);
+    str_def1 = stringNew(bigbuf);
+
         /* Insert function header */
     sarrayParseRange(sa1, newstart, &actstart, &end, &newstart, "--", 0);
     sarrayAppendRange(sa4, sa1, actstart, end);
@@ -547,11 +566,15 @@ SEL     *sel;
         sarrayAppendRange(sa4, sa1, argstart, argend);
 
             /* Declare and define wplsN args, as necessary */
-        if ((sel = selaGetSel(sela, i)) == NULL)
+        if ((sel = selaGetSel(sela, i)) == NULL) {
+            sarrayDestroy(&sa1);
+            sarrayDestroy(&sa2);
+            sarrayDestroy(&sa3);
+            sarrayDestroy(&sa4);
             return ERROR_INT("sel not returned", procName, 1);
-        if ((sa5 = sarrayMakeWplsCode(sel)) == NULL)
-            return ERROR_INT("sa5 not made", procName, 1);
-        sarrayConcatenate(sa4, sa5);
+        }
+        sa5 = sarrayMakeWplsCode(sel);
+        sarrayJoin(sa4, sa5);
         sarrayDestroy(&sa5);
 
             /* Make sure sel has at least one hit */
@@ -566,7 +589,8 @@ SEL     *sel;
             }
         }
         if (nhits == 0) {
-            linestr = stringNew("    fprintf(stderr, \"Error in HMT: no hits in sel!\\n\");\n}\n\n");
+            linestr = stringNew("    "
+                "lept_stderr(\"Error in HMT: no hits in sel!\\n\");\n}\n\n");
             sarrayAddString(sa4, linestr, L_INSERT);
             continue;
         }
@@ -575,9 +599,14 @@ SEL     *sel;
         sarrayAppendRange(sa4, sa1, loopstart, loopend);
 
             /* Insert barrel-op code for *dptr */
-        if ((sa6 = sarrayMakeInnerLoopDWACode(sel, nhits, nmisses)) == NULL)
+        if ((sa6 = sarrayMakeInnerLoopDWACode(sel, nhits, nmisses)) == NULL) {
+            sarrayDestroy(&sa1);
+            sarrayDestroy(&sa2);
+            sarrayDestroy(&sa3);
+            sarrayDestroy(&sa4);
             return ERROR_INT("sa6 not made", procName, 1);
-        sarrayConcatenate(sa4, sa6);
+        }
+        sarrayJoin(sa4, sa6);
         sarrayDestroy(&sa6);
 
             /* Finish the function code */
@@ -585,11 +614,10 @@ SEL     *sel;
     }
 
         /* Output to file */
-    if ((filestr = sarrayToString(sa4, 1)) == NULL)
-        return ERROR_INT("filestr from sa4 not made", procName, 1);
+    filestr = sarrayToString(sa4, 1);
     nbytes = strlen(filestr);
     if (filename)
-        sprintf(bigbuf, "%slow.%d.c", filename, fileindex);
+        snprintf(bigbuf, L_BUF_SIZE, "%slow.%d.c", filename, fileindex);
     else
         sprintf(bigbuf, "%slow.%d.c", OUTROOT, fileindex);
     l_binaryWrite(bigbuf, "w", filestr, nbytes);
@@ -597,8 +625,7 @@ SEL     *sel;
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
     sarrayDestroy(&sa4);
-    FREE(filestr);
-
+    LEPT_FREE(filestr);
     return 0;
 }
 
@@ -608,7 +635,7 @@ SEL     *sel;
  *                            Helper code for sel                           *
  *--------------------------------------------------------------------------*/
 /*!
- *  sarrayMakeWplsCode()
+ * \brief   sarrayMakeWplsCode()
  */
 static SARRAY *
 sarrayMakeWplsCode(SEL  *sel)
@@ -636,39 +663,38 @@ SARRAY  *sa;
         ymax = 31;
     }
 
-    if ((sa = sarrayCreate(0)) == NULL)
-        return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
+    sa = sarrayCreate(0);
 
         /* Declarations */
     if (ymax > 4)
-        sarrayAddString(sa, wpldecls[2], 1);
+        sarrayAddString(sa, wpldecls[2], L_COPY);
     if (ymax > 8)
-        sarrayAddString(sa, wpldecls[6], 1);
+        sarrayAddString(sa, wpldecls[6], L_COPY);
     if (ymax > 12)
-        sarrayAddString(sa, wpldecls[10], 1);
+        sarrayAddString(sa, wpldecls[10], L_COPY);
     if (ymax > 16)
-        sarrayAddString(sa, wpldecls[14], 1);
+        sarrayAddString(sa, wpldecls[14], L_COPY);
     if (ymax > 20)
-        sarrayAddString(sa, wpldecls[18], 1);
+        sarrayAddString(sa, wpldecls[18], L_COPY);
     if (ymax > 24)
-        sarrayAddString(sa, wpldecls[22], 1);
+        sarrayAddString(sa, wpldecls[22], L_COPY);
     if (ymax > 28)
-        sarrayAddString(sa, wpldecls[26], 1);
+        sarrayAddString(sa, wpldecls[26], L_COPY);
     if (ymax > 1)
-        sarrayAddString(sa, wpldecls[ymax - 2], 1);
+        sarrayAddString(sa, wpldecls[ymax - 2], L_COPY);
 
-    sarrayAddString(sa, emptystring, 1);
+    sarrayAddString(sa, emptystring, L_COPY);
 
         /* Definitions */
     for (i = 2; i <= ymax; i++)
-        sarrayAddString(sa, wpldefs[i - 2], 1);
+        sarrayAddString(sa, wpldefs[i - 2], L_COPY);
 
     return sa;
 }
 
 
 /*!
- *  sarrayMakeInnerLoopDWACode()
+ * \brief   sarrayMakeInnerLoopDWACode()
  */
 static SARRAY *
 sarrayMakeInnerLoopDWACode(SEL     *sel,
@@ -686,9 +712,7 @@ SARRAY  *sa;
     if (!sel)
         return (SARRAY *)ERROR_PTR("sel not defined", procName, NULL);
 
-    if ((sa = sarrayCreate(0)) == NULL)
-        return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
-
+    sa = sarrayCreate(0);
     ntot = nhits + nmisses;
     nfound = 0;
     for (i = 0; i < sel->sy; i++) {
@@ -711,8 +735,8 @@ SARRAY  *sa;
                     sprintf(bigbuf, "                    %s %s", string, land);
                 else  /* nfound == ntot */
                     sprintf(bigbuf, "                    %s;", string);
-                sarrayAddString(sa, bigbuf, 1);
-                FREE(string);
+                sarrayAddString(sa, bigbuf, L_COPY);
+                LEPT_FREE(string);
             }
         }
     }
@@ -722,7 +746,7 @@ SARRAY  *sa;
 
 
 /*!
- *  makeBarrelshiftString()
+ * \brief   makeBarrelshiftString()
  */
 static char *
 makeBarrelshiftString(l_int32  delx,    /* j - cx */
